@@ -1,19 +1,12 @@
-# We need to ensure that:
-# - `import pcbnew` provides the original pcbnew module
-# - `from pcbnewTransition` import pcbnew import the patched module. Therefore,
-# we cannot patch the original module. Instead, we have to trick python to load
-# a module twice. Therefore, let's delete pcbnew from sys.modules, import it, patch it and delete from sys.modules
 import sys
 
 originalPcbnew = None
 try:
     originalPcbnew = sys.modules["pcbnew"]
-    del sys.modules["pcbnew"]
 except KeyError:
     pass
 
-from . import pcbnew
-import types
+import pcbnew
 
 def getVersion():
     try:
@@ -23,6 +16,33 @@ def getVersion():
         # KiCAD 5 does not have such function, assume it version 5.something
         return 5, 0
 
+KICAD_VERSION = getVersion()
+
+def isV6(version=KICAD_VERSION):
+    if version[0] == 5 and version[1] == 99:
+        return True
+    return version[0] == 6
+
+def isV7(version=KICAD_VERSION):
+    if version[0] == 6 and version[1] == 99:
+        return True
+    return version[0] == 7
+
+def isV8(version=KICAD_VERSION):
+    if version[0] == 7 and version[1] == 99:
+        return True
+    return version[0] == 8
+
+if isV6(KICAD_VERSION):
+    # There are big breaking changes between 6 -> 7.
+    # We need to ensure that:
+    # - `import pcbnew` provides the original pcbnew module
+    # - `from pcbnewTransition` import pcbnew import the patched module. Therefore,
+    # we cannot patch the original module. Instead, we have to trick python to load
+    # a module twice. Therefore, let's delete pcbnew from sys.modules, import it, patch it and delete from sys.modules
+    del sys.modules["pcbnew"]
+    from . import pcbnew
+
 def boardGetProperties(self):
     return {}
 
@@ -72,6 +92,12 @@ def patchRotate(item):
             newGetDrawRotation = lambda self: pcbnew.EDA_ANGLE(originalGetDrawRotation(self), pcbnew.TENTHS_OF_A_DEGREE_T)
             setattr(newGetDrawRotation, "patched", True)
             item.GetDrawRotation = newGetDrawRotation
+    if hasattr(item, "GetTextAngle"):
+        originalGetTextAngle = item.GetTextAngle
+        if not getattr(originalGetTextAngle, "patched", False):
+            newGetTextAngle = lambda self: pcbnew.EDA_ANGLE(originalGetTextAngle(self), pcbnew.TENTHS_OF_A_DEGREE_T)
+            setattr(newGetTextAngle, "patched", True)
+            item.GetTextAngle = newGetTextAngle
     if hasattr(item, "SetTextAngle"):
         originalSetTextAngle = item.SetTextAngle
         if not getattr(originalSetTextAngle, "patched", False):
@@ -84,25 +110,18 @@ def patchRotate(item):
             newSetHatchOrientation = lambda self, angle: originalSetHatchOrientation(self, angle.AsTenthsOfADegree())
             setattr(newSetHatchOrientation, "patched", True)
             item.SetHatchOrientation = newSetHatchOrientation
+    if hasattr(item, "SetArcAngleAndEnd"):
+        originalSetArcAngleAndEnd = item.SetArcAngleAndEnd
+        if not getattr(originalSetArcAngleAndEnd, "patched", False):
+            newSetArcAngleAndEnd = lambda self, angle, check_neg: originalSetArcAngleAndEnd(self, angle.AsTenthsOfADegree(), check_neg)
+            setattr(newSetArcAngleAndEnd, "patched", True)
+            item.SetArcAngleAndEnd = newSetArcAngleAndEnd
 
 def pathGetItemDescription(item):
     if hasattr(item, "GetSelectMenuText") and not hasattr(item, "GetItemDescription"):
         setattr(item, "GetItemDescription", getattr(item, "GetSelectMenuText"))
 
-
-KICAD_VERSION = getVersion()
-
-def isV6(version=KICAD_VERSION):
-    if version[0] == 5 and version[1] == 99:
-        return True
-    return version[0] == 6
-
-def isV7(version=KICAD_VERSION):
-    if version[0] == 6 and version[1] == 99:
-        return True
-    return version[0] == 7
-
-if not isV6(KICAD_VERSION) and not isV7(KICAD_VERSION):
+if not isV6(KICAD_VERSION) and not isV7(KICAD_VERSION) and not isV8(KICAD_VERSION):
     # Introduce new functions
     pcbnew.NewBoard = NewBoard
 
@@ -158,7 +177,7 @@ except ImportError:
     pcbnew.RADIANS_T = EDA_ANGLE_T.RADIANS_T
     pcbnew.TENTHS_OF_A_DEGREE_T = EDA_ANGLE_T.TENTHS_OF_A_DEGREE_T
 
-if not isV7(KICAD_VERSION):
+if not isV7(KICAD_VERSION) and not isV8(KICAD_VERSION):
     # VECTOR2I & wxPoint
     class _transition_VECTOR2I(pcbnew.wxPoint):
         def __init__(self, *args, **kwargs):
@@ -215,11 +234,16 @@ if not isV7(KICAD_VERSION):
     originalSetSize = pcbnew.PAD.SetSize
     pcbnew.PAD.SetSize = lambda self, size: originalSetSize(self, pcbnew.wxSize(size[0], size[1]))
 
-# We need to ensure that the original pcbnew is not modified
-try:
-    del sys.modules["pcbnew"]
-    if originalPcbnew is not None:
-        sys.modules["pcbnew"] = originalPcbnew
-except KeyError:
-    pass
+# There are some incompatibilites that cannot be monkeypatched in a right way;
+# let's export them as new types:
+pcbnew.FIELD_TYPE = pcbnew.PCB_FIELD if isV8() else pcbnew.FP_TEXT
+
+if isV6():
+    # We need to ensure that the original pcbnew is not modified
+    try:
+        del sys.modules["pcbnew"]
+        if originalPcbnew is not None:
+            sys.modules["pcbnew"] = originalPcbnew
+    except KeyError:
+        pass
 
